@@ -5,7 +5,7 @@ local format = require "luacheck.format"
 local utils = require "luacheck.utils"
 
 local luacheck = {
-   _VERSION = "0.13.0"
+   _VERSION = "0.15.0"
 }
 
 local function raw_validate_options(fname, opts)
@@ -114,7 +114,8 @@ function luacheck.check_files(files, opts)
    local srcs = {}
 
    for i, file in ipairs(files) do
-      srcs[i] = utils.read_file(file) or {fatal = "I/O"}
+      local src, err = utils.read_file(file)
+      srcs[i] = src or {fatal = "I/O", msg = err}
    end
 
    return luacheck.check_strings(srcs, opts)
